<?php

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    wishlist-dot-com-for-woocommerce
 * @subpackage wishlist-dot-com-for-woocommerce/includes
 * @author     WishList.com, Inc. <plugins@wishlist.com>
 */
class Wishlist_Dot_Com_For_Woocommerce {

    /**
     * The loader that's responsible for maintaining and registering all hooks that power
     * the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Wishlist_Dot_Com_For_Woocommerce_Loader    $loader    Maintains and registers all hooks for the plugin.
     */
    protected $loader;

    /**
     * The unique identifier of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $plugin_name    The string used to uniquely identify this plugin.
     */
    protected $plugin_name;

    /**
     * The current version of the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $version    The current version of the plugin.
     */
    protected $version;

    /**
     * Define the core functionality of the plugin.
     *
     * Set the plugin name and the plugin version that can be used throughout the plugin.
     * Load the dependencies, define the locale, and set the hooks for the admin area and
     * the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function __construct() {
        $this->version = WISHLIST_DOT_COM_FOR_WOOCOMMERCE_VERSION;
        $this->plugin_name = WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG;

        $this->load_dependencies();
        $this->set_locale();
        if(is_admin()){
            $this->define_admin_hooks();
        }
        $this->define_public_hooks();

    }

    /**
     * Load the required dependencies for this plugin.
     *
     * Include the following files that make up the plugin:
     *
     * - Wishlist_Dot_Com_For_Woocommerce_Loader. Orchestrates the hooks of the plugin.
     * - Wishlist_Dot_Com_For_Woocommerce_i18n. Defines internationalization functionality.
     * - Wishlist_Dot_Com_For_Woocommerce_Admin. Defines all hooks for the admin area.
     * - Wishlist_Dot_Com_For_Woocommerce_Public. Defines all hooks for the public side of the site.
     *
     * Create an instance of the loader which will be used to register the hooks
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function load_dependencies() {

        /**
         * Trait with shared logic
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wishlist-dot-com-for-woocommerce-trait.php';

        /**
         * The defining all functions
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wishlist-dot-com-for-woocommerce-functions.php';

        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wishlist-dot-com-for-woocommerce-loader.php';

        /**
         * The class responsible for defining internationalization functionality
         * of the plugin.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wishlist-dot-com-for-woocommerce-i18n.php';

        /**
         * The class responsible for defining all actions that occur in the admin area.
         */
        if(is_admin()){
            require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-wishlist-dot-com-for-woocommerce-admin.php';
        }

        /**
         * The class responsible for defining all actions that occur in the public-facing
         * side of the site.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-wishlist-dot-com-for-woocommerce-public.php';

        /**
         * Exopite Simple Options Framework
         *
         * @link https://github.com/JoeSz/Exopite-Simple-Options-Framework
         * @author Joe Szalai
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/exopite-simple-options/exopite-simple-options-framework-class.php';

        $this->loader = new Wishlist_Dot_Com_For_Woocommerce_Loader();

    }

    /**
     * Define the locale for this plugin for internationalization.
     *
     * Uses the Wishlist_Dot_Com_For_Woocommerce_i18n class in order to set the domain and to register the hook
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function set_locale() {

        $plugin_i18n = new Wishlist_Dot_Com_For_Woocommerce_i18n();

        $this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

    }

    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_admin_hooks() {

        $plugin_admin = new Wishlist_Dot_Com_For_Woocommerce_Admin( $this->get_plugin_name(), $this->get_version() );

        $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
        $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

        // Save/Update plugin options
        $this->loader->add_action( 'init', $plugin_admin, 'create_menu', 999 );

        //filter to adjust items prior to saving
        $this->loader->add_filter( 'exopite_sof_save_menu_options', $plugin_admin, 'adjust_options_prior_to_saving', 999 );

        //filter to adjust items prior to getting them
        //$this->loader->add_filter( 'exopite_sof_menu_get_options', $plugin_admin, 'adjust_options_prior_to_getting', 999 );

        $this->loader->add_action( 'wp_ajax_post_options_to_wlcom_rec_ajax', $plugin_admin, 'post_options_to_wlcom_rec_ajax', 999 );
        $this->loader->add_action( 'wp_ajax_get_product_droppown_rec_ajax', $plugin_admin, 'get_product_droppown_rec_ajax', 999 );

        //product added or updated
        $this->loader->add_action( 'woocommerce_new_product', $plugin_admin, 'add_productid_to_wlcom_queue',31 );
        $this->loader->add_action( 'woocommerce_update_product', $plugin_admin, 'add_productid_to_wlcom_queue',31 );
        $this->loader->add_action( 'woocommerce_new_product_variation', $plugin_admin, 'add_productid_to_wlcom_queue',31 );
        $this->loader->add_action( 'woocommerce_update_product_variation', $plugin_admin, 'add_productid_to_wlcom_queue',31 );

    }

    /**
     * Register all of the hooks related to the public-facing functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_public_hooks() {

        $plugin_public = new Wishlist_Dot_Com_For_Woocommerce_Public( $this->get_plugin_name(), $this->get_version() );

        $options = $plugin_public->return_options();
        $store_uuid = get_option(WISHLIST_DOT_COM_FOR_WOOCOMMERCE_SLUG.'_store_uuid');

        if ( !function_exists('wp_get_current_user') ) {
            include_once(ABSPATH . "wp-includes/pluggable.php");
        }

        //plugin status. don't display if not live, admin user, or if it's a call of wl homepage by admin panel's iframe which is used to get theme's colors using javascript
        if( !isset($_GET['qs_flag_get_theme_style']) &&
            !(
                //must have store_uuid
                !empty($store_uuid) &&
                //admin panal's plugin status must be live or testing
                (
                    str_contains($options['wlcom_plgn_plugin_display_status'],'status_live') ||
                    ($options['wlcom_plgn_plugin_display_status'] == 'status_testing' && current_user_can('edit_pages') )
                )
            )
        ){
            //the WishList isn't live, so remove it from the top menu
            //adjust default top menu that's generated from pages. wp_page_menu, wp_page_menu_args
            $this->loader->add_filter( 'wp_page_menu_args', $plugin_public, 'remove_all_wishlist_pages_from_menu', 999 );

            //stop here, since it's not live or admin login
            return 1;
        }

        //position on product page. note priority
        $positions__product_page = array(
            'wishlist_button_after_add_to_cart_button' => array(
                //can't use woocommerce_after_add_to_cart_button since it doesn't work if there's no add to cart button
                'hook'     => 'woocommerce_single_product_summary',
                'priority' => 31,
                'callback' => 'add2wishlist_button__product_page'
            ),
            'wishlist_button_after_product_info'       => array(
                'hook'     => 'woocommerce_single_product_summary',
                'priority' => 31,
                'callback' => 'add2wishlist_button__product_page'
            ),
            'wishlist_button_after_thumbnails'        => array(
                'hook'     => 'woocommerce_before_single_product_summary',
                'priority' => 21,
                'callback' => 'add2wishlist_button__product_page'
            ),
            'wishlist_button_after_summary'           => array(
                'hook'     => 'woocommerce_after_single_product_summary',
                'priority' => 11,
                'callback' => 'add2wishlist_button__product_page'
            ),
            'wishlist_button_use_javascript'  => array(
                'hook'     => 'woocommerce_after_single_product_summary',
                'priority' => 31,
                'callback' => 'add2wishlist_button__product_page_js_pos'
            ),
            'wishlist_button_use_shortcode'  => array(
                'shortcode'     => 'wishlistdotcom_woo_button_product_page',
                'callback' => 'product_page_button_shortcode'
            ),
        );
        if(!empty($options['wishlist_button_position__product_page']) && !empty($positions__product_page[$options['wishlist_button_position__product_page']])){
            if(isset($positions__product_page[$options['wishlist_button_position__product_page']]['hook'])){
                $this->loader->add_action( $positions__product_page[$options['wishlist_button_position__product_page']]['hook'], $plugin_public, $positions__product_page[$options['wishlist_button_position__product_page']]['callback'], $positions__product_page[$options['wishlist_button_position__product_page']]['priority'] );
            }elseif(isset($positions__product_page[$options['wishlist_button_position__product_page']]['shortcode'])){
                $this->loader->add_shortcode( $positions__product_page[$options['wishlist_button_position__product_page']]['shortcode'], $plugin_public, $positions__product_page[$options['wishlist_button_position__product_page']]['callback']);
            }
        }

        //position on product loop page, note priority
        $positions__product_loop = array(
            'wishlist_button_before_add_to_cart_button' => array(
                'hook'     => 'woocommerce_after_shop_loop_item',
                'priority' => 7,
                'callback' => 'add2wishlist_button__product_loop'
            ),
            'wishlist_button_after_add_to_cart_button'  => array(
                'hook'     => 'woocommerce_after_shop_loop_item',
                'priority' => 15,
                'callback' => 'add2wishlist_button__product_loop'
            ),
            //woocommerce_shop_loop_item_title is likely to be used by the theme
            'wishlist_button_use_javascript'  => array(
                'hook'     => 'woocommerce_shop_loop_item_title',
                'priority' => 31,
                //output hidden span that can be moved and displayed via js
                'callback' => 'add2wishlist_button__product_loop_js_pos'
            ),
            'wishlist_button_use_shortcode'  => array(
                'shortcode'     => 'wishlistdotcom_woo_button_product_loop',
                'callback' => 'poduct_loop_button_shortcode'
            ),
        );
        if(!empty($options['wishlist_button_position__product_loop']) && !empty($positions__product_loop[$options['wishlist_button_position__product_loop']])){
            if(isset($positions__product_loop[$options['wishlist_button_position__product_loop']]['hook'])){
                $this->loader->add_action( $positions__product_loop[$options['wishlist_button_position__product_loop']]['hook'], $plugin_public, $positions__product_loop[$options['wishlist_button_position__product_loop']]['callback'], $positions__product_loop[$options['wishlist_button_position__product_loop']]['priority'] );
            }elseif(isset($positions__product_loop[$options['wishlist_button_position__product_loop']]['shortcode'])){
                $this->loader->add_shortcode( $positions__product_loop[$options['wishlist_button_position__product_loop']]['shortcode'], $plugin_public, $positions__product_loop[$options['wishlist_button_position__product_loop']]['callback']);
            }
        }

        //position on cart page, note priority
        $positions__cart = array(
            'wishlist_button_above_right' => array(
                'hook'     => 'woocommerce_before_cart_table',
                'priority' => 7,
                'callback' => 'add2wishlist_button__cart'
            ),
            'wishlist_button_above_left' => array(
                'hook'     => 'woocommerce_before_cart_table',
                'priority' => 7,
                'callback' => 'add2wishlist_button__cart'
            ),
            'wishlist_button_above_centered' => array(
                'hook'     => 'woocommerce_before_cart_table',
                'priority' => 7,
                'callback' => 'add2wishlist_button__cart'
            ),
            'wishlist_button_below_right'  => array(
                'hook'     => 'woocommerce_after_cart_table',
                'priority' => 15,
                'callback' => 'add2wishlist_button__cart'
            ),
            'wishlist_button_below_left'  => array(
                'hook'     => 'woocommerce_after_cart_table',
                'priority' => 15,
                'callback' => 'add2wishlist_button__cart'
            ),
            'wishlist_button_below_centered'  => array(
                'hook'     => 'woocommerce_after_cart_table',
                'priority' => 15,
                'callback' => 'add2wishlist_button__cart'
            ),
            'wishlist_button_use_javascript'  => array(
                'hook'     => 'woocommerce_after_cart_table',
                'priority' => 31,
                'callback' => 'add2wishlist_button__cart_js_pos'
            ),
            'wishlist_button_use_shortcode'  => array(
                'shortcode'     => 'wishlistdotcom_woo_button_cart',
                'callback' => 'cart_button_shortcode'
            ),
        );
        if(!empty($options['wishlist_button_position__cart']) && !empty($positions__cart[$options['wishlist_button_position__cart']])){
            if(isset($positions__cart[$options['wishlist_button_position__cart']]['hook'])){
                $this->loader->add_action( $positions__cart[$options['wishlist_button_position__cart']]['hook'], $plugin_public, $positions__cart[$options['wishlist_button_position__cart']]['callback'], $positions__cart[$options['wishlist_button_position__cart']]['priority'] );
            }elseif(isset($positions__cart[$options['wishlist_button_position__cart']]['shortcode'])){
                $this->loader->add_shortcode( $positions__cart[$options['wishlist_button_position__cart']]['shortcode'], $plugin_public, $positions__cart[$options['wishlist_button_position__cart']]['callback']);
            }
        }

        //store id, js var, modal html.
        //wp_body_open is not called by all themes. use these instead
        $this->loader->add_action( 'wp_head', $plugin_public, 'global_js_var',31 );

        //add to WishList.com javascript
        $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
        $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );

        //hidden wish data on cart page with each item in the cart
        $this->loader->add_action( 'woocommerce_after_cart_item_name', $plugin_public, 'add2WishList_wish_data_cart',31 );

        //hidden notice on shopping cart page
        $this->loader->add_action( 'woocommerce_before_cart', $plugin_public, 'cart_shipping_address_message_html',31 );

        //hidden wish data on checkout page
        $this->loader->add_action( 'woocommerce_before_checkout_form', $plugin_public, 'add2WishList_wish_data_checkout',31 );

        //hidden notice on checkout form
        $this->loader->add_action( 'woocommerce_before_checkout_shipping_form', $plugin_public, 'checkout_shipping_address_message_html',31 );

        //shortcode for wishlist full homepage
        $this->loader->add_shortcode( 'wishlistdotcom_woo_hp_shortcode', $plugin_public, 'hp_shortcode');
    }

    /**
     * Run the loader to execute all of the hooks with WordPress.
     *
     * @since    1.0.0
     */
    public function run() {
        $this->loader->run();
    }

    /**
     * The name of the plugin used to uniquely identify it within the context of
     * WordPress and to define internationalization functionality.
     *
     * @since     1.0.0
     * @return    string    The name of the plugin.
     */
    public function get_plugin_name() {
        return $this->plugin_name;
    }

    /**
     * The reference to the class that orchestrates the hooks with the plugin.
     *
     * @since     1.0.0
     * @return    Wishlist_Dot_Com_For_Woocommerce_Loader    Orchestrates the hooks of the plugin.
     */
    public function get_loader() {
        return $this->loader;
    }

    /**
     * Retrieve the version number of the plugin.
     *
     * @since     1.0.0
     * @return    string    The version number of the plugin.
     */
    public function get_version() {
        return $this->version;
    }

}
